import { NextFunction, Request, Response } from "express";
import jwt from "jsonwebtoken";
import { AppDataSource } from "../data-source";
import { Post } from "../entity/Post";
import { User } from "../entity/User";

const authenticate = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    const token = req.cookies.jwt;
    if (!token || !jwt.verify(token, process.env.JWT_SECRET)) {
        res.status(401).send("No valid JWT is present");
        return;
    }
    next();
}

const authorizeForPost = async (req: Request, res: Response, next: NextFunction): Promise<void> => {
    const { postId } = req.params;
    const user = (jwt.decode(req.cookies.jwt) as jwt.JwtPayload);
    const userId = user.id;
    const postAuthorId = (await AppDataSource.manager.findOneBy(
        Post,
        { id: Number.parseInt(postId) }
    )).authorId

    if (userId != postAuthorId) {
        res.status(403).send("Not your post");
        return;
    }
    next();
}

export default { authenticate, authorizeForPost };